<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Registry;

use Dedi\SyliusMigrationPlugin\Service\ErrorHandler\ErrorHandlerInterface;

class ErrorHandlerRegistry
{
    private array $errorHandlers = [];

    public function getErrorHandlers(): array
    {
        return $this->errorHandlers;
    }

    public function addErrorHandler(ErrorHandlerInterface $errorHandler): array
    {
        if (!$this->hasErrorHandler($errorHandler->getType())) {
            $this->errorHandlers[$errorHandler->getType()] = $errorHandler;
        }

        return $this->errorHandlers;
    }

    public function getErrorHandler(string $type): ErrorHandlerInterface
    {
        if (!$this->hasErrorHandler($type)) {
            throw new \LogicException(sprintf('ErrorHandler %s not registered', $type));
        }

        return $this->errorHandlers[$type];
    }

    public function hasErrorHandler(string $type): bool
    {
        return array_key_exists($type, $this->errorHandlers);
    }
}
