<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Registry;

use Dedi\SyliusMigrationPlugin\Service\Handler\HandlerInterface;

class HandlerRegistry
{
    private array $handlers = [];

    public function getHandlers(): array
    {
        return $this->handlers;
    }

    public function addHandler(HandlerInterface $handler): array
    {
        if (!$this->hasHandler($handler->getType())) {
            $this->handlers[$handler->getType()] = $handler;
        }

        return $this->handlers;
    }

    public function getHandler(string $type): HandlerInterface
    {
        if (!$this->hasHandler($type)) {
            throw new \LogicException(sprintf('Handler %s not registered', $type));
        }

        return $this->handlers[$type];
    }

    public function hasHandler(string $type): bool
    {
        return array_key_exists($type, $this->handlers);
    }
}
