<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Repository;

use Dedi\SyliusMigrationPlugin\Config\MigrationConfig;
use Dedi\SyliusMigrationPlugin\Definition\TableMappingDefinition;
use Doctrine\DBAL\Connection;

class MappingRepository
{
    private MigrationConfig $config;
    private Connection $connection;

    public function __construct(
        Connection $connection,
        MigrationConfig $config
    ) {
        $this->connection = $connection;
        $this->config = $config;
    }

    public function save(TableMappingDefinition $tableMappingDefinition, array $data): void
    {
        foreach ($tableMappingDefinition->getIdentifierMappingDefinitions() as $identifierMappingDefinition) {
            $destColumnName = $identifierMappingDefinition->getDestColumnName();
            if ($destColumnName === 'id') {
                $destColumnName = 'dest_id';
            }
            $qb = $this->connection->createQueryBuilder();
            $qb->insert($this->config->getTempMappingTableName())
                ->setValue('source_table_name', ':source_table_name')
                ->setValue('source_column_name', ':source_column_name')
                ->setValue('dest_table_name', ':dest_table_name')
                ->setValue('dest_column_name', ':dest_column_name')
                ->setValue('source_id', ':source_id')
                ->setValue('dest_id', ':dest_id')
                ->setParameter('source_table_name', $identifierMappingDefinition->getSourceTableName())
                ->setParameter('source_column_name', $identifierMappingDefinition->getSourceColumnName())
                ->setParameter('dest_table_name', $identifierMappingDefinition->getDestTableName())
                ->setParameter('dest_column_name', $identifierMappingDefinition->getDestColumnName())
                ->setParameter('source_id', $data[$identifierMappingDefinition->getSourceColumnName()])
                ->setParameter('dest_id', $data[$destColumnName]);

            $qb->execute();
        }
    }

    public function getOne(string $sourceTableName, string $sourceColumnName, int $sourceId, ?string $destTableName = null)
    {
        $qb = $this->connection->createQueryBuilder();
        $qb
            ->select('*')
            ->from($this->config->getTempMappingTableName())
            ->andWhere('source_table_name = :sourceTable')
            ->andWhere('source_column_name = :sourceColumn')
            ->andWhere('source_id = :sourceId')
            ->setParameter('sourceTable', $sourceTableName)
            ->setParameter('sourceColumn', $sourceColumnName)
            ->setParameter('sourceId', $sourceId)
            ->setMaxResults(1)
        ;

        if (null !== $destTableName) {
            $qb
                ->andWhere('dest_table_name = :destTable')
                ->setParameter('destTable', $destTableName);
        }

        return $qb->execute()->fetchAssociative();
    }
}
