<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\ErrorHandler;

use Dedi\SyliusMigrationPlugin\Definition\TableMappingDefinition;
use Dedi\SyliusMigrationPlugin\Repository\MappingRepository;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use Exception;

class SaveDuplicateHandler extends AbstractErrorHandler
{
    private Connection $connection;
    private MappingRepository $mappingRepository;

    public function __construct(
        Connection $connection,
        MappingRepository $mappingRepository
    ) {
        $this->connection = $connection;
        $this->mappingRepository = $mappingRepository;
    }

    public function getType(): string
    {
        return 'save_duplicate';
    }

    public function execute(TableMappingDefinition $tableMappingDefinition, array $data): void
    {
        $columns = $this->getConfiguration('columns');

        $qb = $this->connection->createQueryBuilder();
        $qb->select('id')
            ->from($tableMappingDefinition->getTargetTableName(), 'a');

        foreach ($columns as $column) {
            $value = $data[$column];
            $qb->andWhere('a.' . $column . ' = :' . $column)
                ->setParameter($column, $value);
        }

        $id = $qb->execute()->fetchOne();

        if (false !== $id) {
            $data['dest_id'] = $id;
            $this->mappingRepository->save($tableMappingDefinition, $data);
        }
    }

    public function getErrorBubbling(): bool
    {
        return false;
    }

    public function can(Exception $exception): bool
    {
        return $exception instanceof UniqueConstraintViolationException;
    }
}
